require "/scripts/messageutil.lua"
require "/scripts/util.lua"

-- Enables and sets up weight gain.
function setup()
	-- Load up base parameters.
	local basePreferences = root.assetJson("/scripts/player/bigfatties.config:basePreferences")

	-- Save parameters into player file.
	player.setProperty("bigfatties", sb.jsonMerge(sb.jsonMerge(basePreferences, player.getProperty("bigfatties") or {}), {enabled = true}))

	-- Run init.
	init()
end

-- Run on load.
function init()
	-- Handler to check if the GUI enabled this.
	message.setHandler("bigFatties_setup", localHandler(setup))

	-- Handler to check if the GUI disabled this.
	message.setHandler("bigFatties_disable", localHandler(disable))

	-- Handler to check if the GUI reset this.
	message.setHandler("bigFatties_cleanse", localHandler(cleanse))

	-- Setup handler for other entities to directly feed the player.
	message.setHandler("bigFatties_feed", simpleHandler(
		function(baseAmount, instant)
			-- Check if mod is enabled.
			if player.getProperty("bigfatties") then
			preferences = player.getProperty("bigfatties")
				-- Calculate the food amount.
				local foodAmount = baseAmount * preferences.settings.foodValue
				-- Add it to be consumed by the player, and save preferences.
				if foodAmount > 0 then
					if instant then
						if preferences.features.dynamic then
							preferences.tracker.size = math.max(math.min(preferences.tracker.size + foodAmount * foodModifier, (#levels + 1)*100), 0)
						end
						status.modifyResource("health", foodAmount * preferences.settings.digestionHeal)
						status.modifyResource("food", foodAmount * 5)
					else
						preferences.tracker.stomach = preferences.tracker.stomach + foodAmount
					end
				end
				player.setProperty("bigfatties", preferences)
				-- Respond back to the entity.
				return true
			else
				return false
			end
		end
	))

	-- Setup handler for other entities to make the player lose weight.
	message.setHandler("bigFatties_weightLoss", simpleHandler(
		function(baseAmount)
			-- Check if mod is enabled.
			if player.getProperty("bigfatties") then
				-- Check if dynamic size is enabled.
				preferences = player.getProperty("bigfatties")
				if preferences.features.dynamic then
					-- Calculate the food amount.
					local weightLossAmount = baseAmount * preferences.settings.weightLossRate * foodModifier^(1/3)
					-- Add it to be consumed by the player, and save preferences.
					if weightLossAmount > 0 then
						preferences.tracker.size = math.max(math.min(preferences.tracker.size - weightLossAmount, (#levels + 1)*100), 0)
					end
					player.setProperty("bigfatties", preferences)
					-- Respond back to the entity.
					return true
				else
					return false
				end
			else
				return false
			end
		end
	))


	-- Setup handler for other entities to get the player's level, progress, and size.
	message.setHandler("bigFatties_size", simpleHandler(
		function()
			-- Check if mod is enabled.
			if preferences and preferences.enabled then

				local weight = 0
				for i=0,level do
					weight = weight + (i == level and math.min(progress/100, 1) or 1) * (levels[i] and levels[i].massIncrease or 100)
				end

				return {
					level = level,
					progress = (preferences.features.dynamic and progress or 100),
					size = preferences.tracker.size + (preferences.features.dynamic and (preferences.settings.baseSize * 100) or 0),
					weight = {kg = math.floor((weight/2.205) + 0.5), lb = math.floor(weight + 0.5)},
					levelWeight = math.min((preferences.features.dynamic and progress or 100)/100, 1) * (levels[level] and levels[level].massIncrease or 100),
					totalLevelWeight = levels[level] and levels[level].massIncrease or 100
				}
			else
				return {level = 0, progress = 0, size = 0, weight = {kg = 0, lb = 0}, levelWeight = 0, totalLevelWeight = 100}
			end
		end
	))

	-- Load preferences into variable.
	preferences = player.getProperty("bigfatties")

	-- Small wip fix so people don't have to reset (delete this later).
	if preferences and type(preferences.tracker.stomach) == "table" then preferences.tracker.stomach = 0 end

	if preferences and preferences.enabled then

		-- Merge on top of base parameters (Lets new stuff be added without having to clearing all the settings).
		preferences = sb.jsonMerge(root.assetJson("/scripts/player/bigfatties.config:basePreferences"), preferences)

		-- Save preferences again.
		player.setProperty("bigfatties", preferences)

		-- Grab food values from the config.
		baseFoodValues = root.assetJson("/scripts/player/bigfatties.config:foodValues")

		-- Grab drinkable liquids from the config.
		drinkables = root.assetJson("/scripts/player/bigfatties.config:drinkables")

		-- Load up weight/protection/etc for each level.
		levels = root.assetJson("/scripts/player/bigfatties.config:levels")

		-- Load up hitboxs for each weight size.
		sizes = root.assetJson("/scripts/player/bigfattiessizes.config")

		-- Recalculate any left over variables incase we're re-enabling.
		-- Check if dynamic size is enabled.
		if preferences.features.dynamic then
			-- Current level is size/100, rounded down. Progress is how far between '100's the player is.
			level = math.min(math.floor((preferences.tracker.size + preferences.settings.baseSize * 100)/100), 10)
			progress = math.min((preferences.tracker.size + preferences.settings.baseSize * 100) - level * 100, 100)
		else
			-- If the player has dynamic size disabled, use their static size (Increasing the size directly means they could 'cheat' their way higher).
			level = preferences.settings.staticSize
			progress = 100
		end

		oldLevel = nil
		oldProgress = nil

		foodModifier = levels[level] and 100/levels[level].massIncrease or 1
	end
end

-- Run each tick.
function update(dt)
	-- Load up preferences.
	preferences = player.getProperty("bigfatties")

	if preferences and preferences.enabled then
		-- Handler for playing sound.
		status.addEphemeralEffect("bigfattiessoundhandler")

		-- Blobs tend to get stuck in the ground when beaming.
		if not beamedIn and levels[level] and levels[level].kind:find("blob") then
			for i,v in ipairs(status.activeUniqueStatusEffectSummary()) do
				if v[1] == "playerbeamin" then
					mcontroller.translate({0, 2.5})
					beamedIn = true
				end
			end
		end

		-- Check if dynamic size is enabled.
		if preferences.features.dynamic then
			-- Current level is size/100, rounded down. Progress is how far between '100's the player is.
			level = math.min(math.floor((preferences.tracker.size + preferences.settings.baseSize * 100)/100), #levels)
			progress = math.min((preferences.tracker.size + preferences.settings.baseSize * 100) - level * 100, 100)
		else
			-- If the player has dynamic size disabled, use their static size (Increasing the size directly means they could 'cheat' their way higher).
			level = preferences.settings.staticSize
			progress = 100
		end

		stuffed = preferences.features.stuffed
		filled = preferences.features.filled

		-- Check if the player is filled/stuffed.
		bfvpreferences = player.getProperty("bigfattiesvore")
		if bfvpreferences then
			for _,v in ipairs(bfvpreferences.tracker.stomach) do
				stuffed = stuffed or world.entityType(v) == "monster"
				filled = filled or (world.entityType(v) == "npc" or world.entityType(v) == "player")
			end
		end


		-- Increase food values based on settings.
		foodValues = jarray()
		for k,v in pairs(baseFoodValues) do
			foodValues[k] = v * preferences.settings.foodValue
		end

		foodModifier = levels[level] and 100/levels[level].massIncrease or 1

		-- Update status bar progress.
		world.sendEntityMessage(player.id(), "bigFatties_changeProgress", level, progress)

		-- Check if dynamic size is enabled (Yes, we do need two of these blocks).
		if #status.getPersistentEffects("bigfattiesvore") == 0 then
			-- Lose weight when moving.
			if preferences.features.dynamic then
				weightLoss(preferences.settings.weightLossRate, foodModifier, dt)
			end
			-- Consume and digest food.
			detectFood()
			digestFood(preferences.settings.digestionRate, (digestionBoost or 0), preferences.settings.digestionHeal, preferences.settings.gurgleRate, foodModifier, dt)
			-- Check if drinking is enabled.
			if preferences.features.drinking then
				-- Drink liquids.
				drink(dt)
			end
		end

		-- Remove prior bonuses.
		status.clearPersistentEffects("bigfatties")


		if levels[level] then
			-- Check if the player has thick thighs enabled.
			legSize = levels[level].kind
			if sizes[levels[level].kind].hasThickThighs and preferences.features.thickThighs then
				for i,v in ipairs(sizes.list) do
					if legSize == v then
						legSize = sizes.list[i + 1]
						break
					end
				end
			end

			-- Check if the player is in a mech, and cap their size (Using busty to imitate levels that would clip out).
			local mechLevel = level
			local mechBusty = false
			if player.isLounging() and world.entityName(player.loungingIn()) == "modularmech" then
				mechLevel = math.min(level, 6)
				if level > 6 then
					mechBusty = true
					legSize = levels[mechLevel].kind
				end
			end

			-- Check if the player has the right items.
			checkFatItems(levels[mechLevel].kind,
				legSize,
				sizes[levels[mechLevel].kind].hasBusty and (preferences.features.busty or mechBusty),
				sizes[levels[mechLevel].kind].hasStuffed and stuffed,
				sizes[levels[mechLevel].kind].hasFilled and filled
			)

			-- Check if effects is enabled.
			if preferences.features.effects then
				-- Give the player hitbox, bonus stats, and effects based on fatness.
				status.addPersistentEffects("bigfatties", levels[level].statModifiers)

				mcontroller.controlModifiers({
					airJumpModifier = levels[level].movementPenalty,
					groundMovementModifier = levels[level].movementPenalty
				})
				local weight = 0
				for i=0,level do
					weight = weight + (i == level and math.min(progress/100, 1) or 1) * (levels[i] and levels[i].massIncrease or 100)
				end
				-- Average human weight is 137lb/62kg, math is based off of that.
				weight = weight/137
				mcontroller.controlParameters(sb.jsonMerge((((#status.getPersistentEffects("movementAbility") > 0 or #status.getPersistentEffects("weaponMovementAbility") == 0) and status.stat("activeMovementAbilities") == 1) and {} or sizes[levels[level].kind]), sb.jsonMerge(levels[level].movementParameters, {
					mass = mcontroller.baseParameters().mass + weight,
					walkSpeed = mcontroller.baseParameters().walkSpeed * levels[level].movementPenalty,
					runSpeed = mcontroller.baseParameters().runSpeed * levels[level].movementPenalty,
					flySpeed = mcontroller.baseParameters().flySpeed * levels[level].movementPenalty,

					groundForce = mcontroller.baseParameters().groundForce * (mcontroller.baseParameters().mass + weight),
					airForce = mcontroller.baseParameters().airForce * (mcontroller.baseParameters().mass + weight),
					liquidForce = mcontroller.baseParameters().liquidForce * (mcontroller.baseParameters().mass + weight),
					airFriction = mcontroller.baseParameters().airFriction * (mcontroller.baseParameters().mass + weight),
					liquidFriction = mcontroller.baseParameters().liquidFriction * (mcontroller.baseParameters().mass + weight),
					normalGroundFriction = mcontroller.baseParameters().normalGroundFriction * (mcontroller.baseParameters().mass + weight),
					airJumpProfile = {
					  jumpControlForce = mcontroller.baseParameters().airJumpProfile.jumpControlForce * (mcontroller.baseParameters().mass + weight),
					},
					liquidJumpProfile = {
					  jumpSpeed = mcontroller.baseParameters().liquidJumpProfile.jumpSpeed * levels[level].movementPenalty,
					  jumpControlForce = mcontroller.baseParameters().liquidJumpProfile.jumpControlForce * (mcontroller.baseParameters().mass + weight) * levels[level].movementPenalty,
					}
				})))
			end

			-- Check if blob hitbox is enabled, that the player isn't sitting down, they're not using a movement tech, and they aren't eaten.
			if preferences.features.blobHitbox and not player.isLounging() and #status.getPersistentEffects("bigfattiesvore") == 0 then
				-- Solid hitbox for reaching blob body.
				if levels[level].kind:find("blob") then
					world.spawnProjectile("blobhitbox", world.entityPosition(player.id()), player.id(), {0,0}, true)
				end
			end
		elseif preferences.features.thickThighs or preferences.features.busty or stuffed or filled or preferences.clothing.legsarmor or preferences.clothing.chestarmor then
			checkFatItems("", preferences.features.thickThighs and sizes.list[1] or "", preferences.features.busty, stuffed, filled)
		else
			removeFatItems()
		end

		-- Check if the level has changed.
		if level ~= oldLevel then
			-- If they revert back to normal size, cleanup any leftover items. Small fix for blobEnd equipment included.
			if not levels[level] or (levels[oldLevel] and levels[oldLevel].kind == "blobend") then
				removeFatItems()
			end

			-- Play sound to indicate size change.
			world.sendEntityMessage(player.id(), "bigFatties_playSound", "digest", 0.75, math.random(10,15)/10 - level/20)

			-- Update status bar tracker.
			status.addEphemeralEffect("bigfattieslevel"..level)
		end
		-- Keep old levels and progress in variables. Used to detect if any size changes need to be made.
		oldLevel = level
		oldProgress = progress

		debugInfo()
		-- Save preferences.
		player.setProperty("bigfatties", preferences)
	end
end

-- Digest the player's stomach content.
function digestFood(digestionRate, digestionBoost, digestionHeal, gurgleRate, foodModifier, dt)
	-- No need to run this block if we're not digesting in the first place, or we're dead.
	if (digestionRate == 0 and digestionBoost == 0) or preferences.tracker.stomach == 0 or not status.resourcePositive("health") then
		return
	end

	-- Calculate how much food to digest. (1 + 1.66% of food per second)
	local digestAmount = (1 + (preferences.tracker.stomach/60)) * (digestionRate + digestionBoost) * dt

	-- Increase size value, decrease stomach value, increase hunger.
	if preferences.features.dynamic then
		preferences.tracker.size = math.max(math.min(preferences.tracker.size + math.min(preferences.tracker.stomach, digestAmount) * foodModifier, (#levels + 1)*100), 0)
	end
	preferences.tracker.stomach = preferences.tracker.stomach - math.min(preferences.tracker.stomach, digestAmount)
	status.modifyResource("health", digestAmount * digestionHeal)
	status.modifyResource("food", digestAmount * 5)

	-- Roughly every 30 seconds, gurgle (i.e. instantly digest 30 seconds worth of food); occurs faster based on digestion speed and gurgle rate.
	if gurgleRate > 0 and math.random(1, math.floor(30/(digestAmount * gurgleRate) + 0.5)) == 1 then
		digestFood(digestionRate, digestionBoost, digestionHeal, 0, foodModifier, dt * 30)
		world.sendEntityMessage(player.id(), "bigFatties_playSound", "digest", 0.75, math.random(10,15)/10 - level/20)
	end
end

-- Digest any food in the player's stomach at a given speed. Modifiers default to 1.
-- digestionBoost significantly increases food digested, and increases gurgle chance.
-- gurgleRate is the average number of gurgles (instantly consume a minute worth of food) per minute.
function weightLoss(weightLossRate, foodModifier, dt)
	-- No need to run this block if we're still digesting food, or have weight loss disabled.
	if weightLossRate == 0 then
		return
	end

	weightLossRate = weightLossRate
	local weightLossAmount = weightLossRate * foodModifier^(1/3) * dt

	-- Check if the player is moving around.
	if mcontroller.onGround() then
		-- If the player is walking.
		if mcontroller.walking() then
			-- Subtract lost amount from size.
			preferences.tracker.size = math.max(preferences.tracker.size - weightLossAmount/16, 0)
		end

		-- If the player is running.
		if mcontroller.running() then
			-- Same as walking, but more.
			preferences.tracker.size = math.max(preferences.tracker.size - weightLossAmount/8, 0)
		end
	end

	-- If the player is jumping.
	if mcontroller.jumping() then
		-- Same as walking, but even more.
		preferences.tracker.size = math.max(preferences.tracker.size - weightLossAmount, 0)
	end
end

-- Gain weight by drinking liquid in front of the player's mouth.
function drink(dt)
	-- More accurately calculate where the player's mouth is and what liquid is in front of it.
	local mouthPosition = {world.entityMouthPosition(player.id())[1], world.entityMouthPosition(player.id())[2] + 1 - (mcontroller.crouching() and 1 or 0)}
	local mouthLiquid = world.liquidAt(mouthPosition)

	-- Space out 'drinks', otherwise they'll happen every script update.
	drinkCooldown = math.max((drinkCooldown or 0) - dt, 0)

	-- Check if drinking isn't on cooldown.
	if drinkCooldown == 0 then
		-- Check if there is liquid in front of the player's mouth, and if it is milk/jelly/lard/chocolate.
		if mouthLiquid and drinkables[root.liquidName(mouthLiquid[1])] then
			-- Remove liquid at the player's mouth, and store how much liquid was removed.
			local consumedLiquid = world.destroyLiquid(mouthPosition)

			-- Check if liquid was removed.
			if consumedLiquid then
				-- Increase drink counter by one, up to a maximum of twenty.
				drinkCounter = math.min((drinkCounter or 0)+ 1, 20)

				-- Add to player's stomach based on liquid consumed.
				preferences.tracker.stomach = preferences.tracker.stomach + drinkables[root.liquidName(consumedLiquid[1])] * consumedLiquid[2]

				-- Reset the drink cooldown, shorter based on how high drinkCounter is.
				drinkCooldown = 1/(1 + drinkCounter/10)

				-- Play drinking sound. Volume increased by amount of liquid consumed.
				world.sendEntityMessage(player.id(), "bigFatties_playSound", "swallow", 0.5 + 0.5 * consumedLiquid[2], math.random(8, 12)/10)
			end
		else
			-- Reset the drink counter if there is nothing to drink.
			drinkCounter = 0
		end
	end
end

-- Check if statuses given by food items have had their duration refreshed/increased (Used to detect eating).
function detectFood()
	-- We don't know if the player still has digestion tonics active.
	digestionBoost = 0

	-- We don't know if the tracking icon has been removed by accident.
	local tracking = false

	-- Cycle through all active status effects, check if any match ones that are in foodValues.
	for i,v in ipairs(status.activeUniqueStatusEffectSummary()) do
			if foodValues[v[1]] then
				-- If any of the statuses have had their duration increased, the player has eaten an item.
				if not preferences.tracker.statusEffects[v[1]] or v[2] >= preferences.tracker.statusEffects[v[1]] then
					-- Consume an amount of food based on item eaten.
					preferences.tracker.stomach = preferences.tracker.stomach + foodValues[v[1]]
				end
				-- Set the duration tracker to the new, higher value.
				preferences.tracker.statusEffects[v[1]] = v[2]
			end

			-- If the player had a distilled digestion tonic, give them a digestion boost.
			if v[1] == "digesting" then
				digestionBoost = 5
			end
		-- Check if we have a tracking icon.
		if string.find(v[1], "bigfattieslevel") then
			tracking = true
		end
	end

	-- If it's gotten removed by accident, add it back.
	if not tracking then
		status.addEphemeralEffect("bigfattieslevel"..level)
	end
end

-- Check if the player has the right items equipped and remove any bugged script items.
function checkFatItems(chestSize, legSize, isBusty, isStuffed, isFilled)
	if preferences.features.equip then
		-- Decide which variant to check for; priority is filled > stuffed > busty.
		local variant = isFilled and "filled" or (isStuffed and "stuffed" or (isBusty and "busty" or ""))

		-- Check if the player is holding a script generated item.
		if player.swapSlotItem() and player.swapSlotItem().parameters and player.swapSlotItem().parameters.isFatItem then
			-- Bye bye item.
			player.setSwapSlotItem(nil)
			-- Give back the item.
			equipFatItems(chestSize, legSize, variant)
		end

		-- Check that they have something equipped and it is the right size.
		if
			not player.equippedItem("legsCosmetic") or
			not player.equippedItem("legsCosmetic").name:find(legSize) or
			not player.equippedItem("legsCosmetic").parameters.isFatItem or
			not preferences.clothing.legsarmor and player.equippedItem("legsCosmetic").parameters.isFatItemClothing or
			preferences.clothing.legsarmor and player.equippedItem("legsCosmetic").parameters.isFatItemClothing and not preferences.clothing.legsarmor.name:find(player.equippedItem("legsCosmetic").parameters.baseName) or
			preferences.clothing.legsarmor and not player.equippedItem("legsCosmetic").parameters.isFatItemClothing and not player.equippedItem("legsCosmetic").parameters.skipCheck
		then
			equipFatItems(chestSize, legSize, variant)
		end

		if
			not player.equippedItem("chestCosmetic") or
			not player.equippedItem("chestCosmetic").name:find(chestSize) or
			not player.equippedItem("chestCosmetic").parameters.isFatItem or
			not preferences.clothing.chestarmor and player.equippedItem("chestCosmetic").parameters.isFatItemClothing or
			preferences.clothing.chestarmor and player.equippedItem("chestCosmetic").parameters.isFatItemClothing and not preferences.clothing.chestarmor.name:find(player.equippedItem("chestCosmetic").parameters.baseName) or
			preferences.clothing.chestarmor and not player.equippedItem("chestCosmetic").parameters.isFatItemClothing and not player.equippedItem("chestCosmetic").parameters.skipCheck or
			(variant == "" and (
				player.equippedItem("chestCosmetic").name:find("busty") or
				player.equippedItem("chestCosmetic").name:find("stuffed") or
				player.equippedItem("chestCosmetic").name:find("filled")
			)) or
			(variant ~= "" and not player.equippedItem("chestCosmetic").name:find(variant))
		then
			equipFatItems(chestSize, legSize, variant)
		end

		if legSize == "" or chestSize == "" then
			for _,val in ipairs({"legs", "chest"}) do
				if player.equippedItem(val.."Cosmetic") then
					for _,v in ipairs(sizes.list) do
						if player.equippedItem(val.."Cosmetic").name:find(v) then
							equipFatItems(chestSize, legSize, variant)
							break
						end
					end
				end
			end
		end
	else
		removeFatItems()
	end
end

-- Equip the player with fat items.
function equipFatItems(chestSize, legSize, variant, availableSizes)
	removeFatItems()
	-- Setup base parameters for item.
	local legsItem = {name = legSize..player.species():lower().."legs", count=1}
	local chestItem = {name = chestSize..variant..player.species():lower().."chest", count=1}

	-- Grab player directives.
	local bodyDirectives = getBodyDirectives()

	-- Flag item as spawned by the script and apply directives.
	legsItem.parameters = {directives = bodyDirectives, price = 0, isFatItem = true, rarity = "essential"}
	chestItem.parameters = {directives = bodyDirectives, price = 0, isFatItem = true, rarity = "essential"}

	if preferences.clothing.legsarmor then
		local legsItemName = preferences.clothing.legsarmor.name
		if legsItemName then
			for _,v in ipairs(sizes.list) do
				legsItemName = legsItemName:gsub(v, "")
			end
		end
		if pcall(root.itemType, legSize..legsItemName) then
			legsItem = {name = legSize..legsItemName, count=1, parameters = sb.jsonMerge(preferences.clothing.legsarmor.parameters, {price = 0, baseName = legsItemName, isFatItem = true, isFatItemClothing = true, rarity = "essential"})}
		else
			legsItem.parameters.skipCheck = true
		end
	end

	if preferences.clothing.chestarmor then
		local chestItemName = preferences.clothing.chestarmor.name
		if chestItemName then
			for _,v in ipairs(sizes.list) do
				chestItemName = chestItemName:gsub(v, "")
			end
		end
		if pcall(root.itemType, chestSize..chestItemName) then
			chestItem = {name = chestSize..chestItemName, count=1, parameters = sb.jsonMerge(preferences.clothing.chestarmor.parameters, {price = 0, baseName = chestItemName, isFatItem = true, isFatItemClothing = true, rarity = "essential"})}
		else
			chestItem.parameters.skipCheck = true
		end
	end
	-- Equip fat pants, return the original to the player.
	if pcall(root.itemType, legsItem.name) then
		if
			-- Check if they are wearing an item.
			player.equippedItem("legsCosmetic") and not
			-- Check if that item is NOT an item generated by this script.
			player.equippedItem("legsCosmetic").parameters.isFatItem and not
			-- Check if that item is NOT the right size.
			(player.equippedItem("legsCosmetic").name:find(legSize) or
			(root.itemConfig(player.equippedItem("legsCosmetic").name).config.shortdescription or ""):find(legSize))
		then
			-- Give them the item back (It's likely to be deleted in the next block).
			player.giveItem(player.equippedItem("legsCosmetic"))
		end
		if not
			-- Check if they're they're not wearing an item.
			player.equippedItem("legsCosmetic") or not
			-- Check if that item is also not the right size.
			(player.equippedItem("legsCosmetic").name:find(legSize) or
			(root.itemConfig(player.equippedItem("legsCosmetic").name).config.shortdescription or ""):find(legSize))
		then
			-- Replace the item.
			player.setEquippedItem("legsCosmetic", legsItem)
		end
	end

	-- Same, but for chest items.
	if pcall(root.itemType, chestItem.name) then
		if
			player.equippedItem("chestCosmetic") and not
			player.equippedItem("chestCosmetic").parameters.isFatItem and not
			(player.equippedItem("chestCosmetic").name:find(chestSize) or
			(root.itemConfig(player.equippedItem("chestCosmetic").name).config.shortdescription or ""):find(chestSize))
		then
			player.giveItem(player.equippedItem("chestCosmetic"))
		end
		if not
			player.equippedItem("chestCosmetic") or not
			(player.equippedItem("chestCosmetic").name:find(chestSize) or
			(root.itemConfig(player.equippedItem("chestCosmetic").name).config.shortdescription or ""):find(chestSize))
		then
			player.setEquippedItem("chestCosmetic", chestItem)
		end
	end
end

-- Remove all items dynamically generated by this script.
function removeFatItems()
	-- Remove from the inventory.
	if player.equippedItem("legsCosmetic") and player.equippedItem("legsCosmetic").parameters.isFatItem then
			player.setEquippedItem("legsCosmetic", nil)
	end
	if player.equippedItem("chestCosmetic") and player.equippedItem("chestCosmetic").parameters.isFatItem then
			player.setEquippedItem("chestCosmetic", nil)
	end
end

-- Returns the player's base body directives.
function getBodyDirectives()
	local bodyDirectives = ""
	for _,v in ipairs(world.entityPortrait(player.id(), "fullnude")) do
		if string.find(v.image, "body.png") then
			bodyDirectives = string.sub(v.image,(string.find(v.image, "?")))
			break
		end
	end
	return bodyDirectives
end


-- Disables the mod.
function disable()
	if preferences then
		-- Get rid of status bar tracking icons.
		world.sendEntityMessage(player.id(), "bigFatties_expire")

		-- Reset preferences (settings/tracking/etc).
		preferences.enabled = false
		player.setProperty("bigfatties", preferences)

		-- Reset level/progress variables.
		level = nil
		progress = nil
		oldLevel = nil
		oldProgress = nil

		-- Clear buffs.
		status.clearPersistentEffects("bigfatties")

		-- Remove any fat items left over.
		removeFatItems()

		-- Do the same for other parts of the mod.
		world.sendEntityMessage(player.id(), "bigFattiesVore_disable")
	end
end

-- Cleanses all mod settings from the player's save.
function cleanse()
	disable()
	player.setProperty("bigfatties", nil)

	-- Do the same for other parts of the mod.
	world.sendEntityMessage(player.id(), "bigFattiesVore_cleanse")
end

-- Print statstics into the debug menu.
function debugInfo()
	sb.setLogMap("^#b8eb00;BigFatties_Level", level.." | "..(math.floor(progress * 100 + 0.5)/100).."%%")
	sb.setLogMap("^#b8eb00;BigFatties_DigestionRate", math.floor(preferences.settings.digestionRate).. " | "..math.floor(digestionBoost or 0))
	sb.setLogMap("^#b8eb00;BigFatties_Stomach", math.floor(preferences.tracker.stomach).."lb")
end
